/* Copyright (c) 2005 David Hammerton - All rights reserved
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "debug.h"
#include "tunesbrowser.h"

#define DEFAULT_DEBUG_CHANNEL "donate"

static GtkWindow *donatenag_win;
static GtkCheckButton *donate_showonstart;

static void on_donate_show_toggled(GtkToggleButton *togglebutton,
                                   __UNUSED__ gpointer user_data)
{
    pref_set_showdonate(gtk_toggle_button_get_active(togglebutton));
}

static void on_donate_donatenow(__UNUSED__ GtkButton *button, __UNUSED__ gpointer user_data)
{
    gtk_widget_hide(GTK_WIDGET(donatenag_win));
    launch_web_url("http://crazney.net/programs/itunes/help.html?from=tunesbrowser");
}

static void on_donate_close(__UNUSED__ GtkButton *button, __UNUSED__ gpointer user_data)
{
    gtk_widget_hide(GTK_WIDGET(donatenag_win));
}

void donate_show()
{
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(donate_showonstart),
                                 pref_get_showdonate());

    gtk_widget_show_all(GTK_WIDGET(donatenag_win));
}

void donate_init(GladeXML *xml)
{
    GtkButton *donate_donatenow;
    GtkButton *donate_close;

    donatenag_win = GTK_WINDOW(glade_xml_get_widget(xml, "donatenag_win"));

    donate_showonstart = GTK_CHECK_BUTTON(glade_xml_get_widget(xml, "donate_showonstart"));
    g_signal_connect(G_OBJECT(donate_showonstart), "toggled",
                     G_CALLBACK(on_donate_show_toggled), NULL);

    donate_donatenow = GTK_BUTTON(glade_xml_get_widget(xml, "donate_donatenow"));
    donate_close = GTK_BUTTON(glade_xml_get_widget(xml, "donate_close"));

    g_signal_connect(G_OBJECT(donate_close), "clicked",
                     G_CALLBACK(on_donate_close), NULL);
    g_signal_connect(G_OBJECT(donate_donatenow), "clicked",
                     G_CALLBACK(on_donate_donatenow), NULL);

    /* make the close button have the focus */
    gtk_widget_grab_focus(GTK_WIDGET(donate_donatenow));
}

